/*
 * FCKeditor - The text editor for Internet - http://www.fckeditor.net
 * Copyright (C) 2003-2007 Frederico Caldeira Knabben
 * 
 * == BEGIN LICENSE ==
 * 
 * Licensed under the terms of any of the following licenses at your
 * choice:
 * 
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 * 
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 * 
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * == END LICENSE ==
 */
package com.fredck.fckeditor.connector;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.filefilter.DirectoryFileFilter;
import org.apache.commons.io.filefilter.FileFileFilter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.fredck.fckeditor.ConfigurationHandler;
import com.fredck.fckeditor.tool.Utils;

/**
 * Servlet to upload and browse files.<br>
 * 
 * This servlet accepts 4 commands used to retrieve and create files and folders from a server directory. The allowed
 * commands are:
 * <ul>
 * <li>GetFolders: Retrive the list of directory under the current folder
 * <li>GetFoldersAndFiles: Retrive the list of files and directory under the current folder
 * <li>CreateFolder: Create a new directory under the current folder
 * <li>FileUpload: Send a new file to the server (must be sent with a POST)
 * </ul>
 * 
 * @version $Id: ConnectorServlet.java 1312 2008-01-14 22:41:41Z mosipov $
 */
public class ConnectorServlet extends HttpServlet {

    private static final long serialVersionUID = -5742008970929377161L;
    private final Logger logger = LoggerFactory.getLogger(ConnectorServlet.class); 

    
    /**
     * Initialize the servlet.<br>
     * Retrieve from the servlet configuration the "baseDir" which is the root of the file repository.
     */
    public void init() throws ServletException {
	if (getInitParameter("baseDir") != null)
	    ConfigurationHandler.setBaseDir(getInitParameter("baseDir"));
	String realBaseDir = getServletContext().getRealPath(ConfigurationHandler.getBaseDir());
	File baseFile = new File(realBaseDir);
	if (!baseFile.exists()) {
	    baseFile.mkdirs();
	}
	logger.info("*** Connector Servlet initialized!");
    }

    /**
     * Manage the Get requests (GetFolders, GetFoldersAndFiles, CreateFolder).<br>
     * 
     * The servlet accepts commands sent in the following format:<br>
     * connector?Command=CommandName&Type=ResourceType&CurrentFolder=FolderPath<br>
     * <br>
     * It execute the command and then return the results to the client in XML format.
     * 
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
	logger.debug("Entered #doGet.");
	
	response.setCharacterEncoding("UTF-8");
	response.setContentType("application/xml; charset=UTF-8");
	response.setHeader("Cache-Control", "no-cache");
	PrintWriter out = response.getWriter();

	String commandStr = request.getParameter("Command");
	String typeStr = request.getParameter("Type");
	String currentFolderStr = request.getParameter("CurrentFolder");

	// TODO untersuchen wie es vom Res Browser kommt
	String currentPath = ConfigurationHandler.getBaseDir() + "/" + typeStr + currentFolderStr;
	currentPath = Utils.replaceAll(currentPath, "//", "/");
	String currentDirPath = getServletContext().getRealPath(currentPath);

	File currentDir = new File(currentDirPath);
	if (!currentDir.exists()) {
	    currentDir.mkdir();
	}

	Document document = null;
	try {
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    document = builder.newDocument();
	} catch (ParserConfigurationException pce) {
	    logger.error("Error while parsing DOM: " + pce.getLocalizedMessage(), pce);
	}

	Node root = CreateCommonXml(document, commandStr, typeStr, currentFolderStr, request.getContextPath()
		+ currentPath);
	logger.debug("Command = " + commandStr);
	
	if (commandStr.equals("GetFolders")) {
	    getFolders(currentDir, root, document);
	} else if (commandStr.equals("GetFoldersAndFiles")) {
	    getFolders(currentDir, root, document);
	    getFiles(currentDir, root, document);
	} else if (commandStr.equals("CreateFolder")) {
	    String newFolderStr = request.getParameter("NewFolderName");
	    File newFolder = new File(currentDir, newFolderStr);
	    String retValue = "110";

	    if (newFolder.exists()) {
		retValue = "101";
	    } else {
		try {
		    boolean dirCreated = newFolder.mkdir();
		    if (dirCreated)
			retValue = "0";
		    else
			retValue = "102";
		} catch (SecurityException sex) {
		    retValue = "103";
		}

	    }
	    setCreateFolderResponse(retValue, root, document);
	}

	document.getDocumentElement().normalize();
	try {
	    TransformerFactory tFactory = TransformerFactory.newInstance();
	    Transformer transformer = tFactory.newTransformer();

	    DOMSource source = new DOMSource(document);

	    StreamResult result = new StreamResult(out);
	    transformer.transform(source, result);
	    /*
	     * if (debug) { StreamResult dbgResult = new StreamResult(System.out); transformer.transform(source,
	     * dbgResult); System.out.println(""); System.out.println("--- END DOGET ---"); }
	     */
	} catch (Exception ex) {
	    logger.error("Error while transforming DOM to HttpServletResponse: " + ex.getMessage(), ex);
	}

	out.flush();
	out.close();
	logger.debug("Successfull ended #doGet!");
    }

    /**
     * Manage the Post requests (FileUpload).<br>
     * 
     * The servlet accepts commands sent in the following format:<br>
     * connector?Command=FileUpload&Type=ResourceType&CurrentFolder=FolderPath<br>
     * <br>
     * It store the file (renaming it in case a file with the same name exists) and then return an HTML file with a
     * javascript command in it.
     * 
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
	logger.debug("Entered #doPost.");

	response.setCharacterEncoding("UTF-8");
	response.setContentType("text/html; charset=UTF-8");
	response.setHeader("Cache-Control", "no-cache");
	PrintWriter out = response.getWriter();

	String commandStr = request.getParameter("Command");
	String typeStr = request.getParameter("Type");
	String currentFolderStr = request.getParameter("CurrentFolder");

	String currentPath = ConfigurationHandler.getBaseDir() + "/" + typeStr + currentFolderStr;
	currentPath = Utils.replaceAll(currentPath, "//", "/");
	String currentDirPath = getServletContext().getRealPath(currentPath);

	logger.debug("current path dir: " + currentDirPath);
	
	String retVal = "0";
	String newName = "";

	if (!commandStr.equals("FileUpload"))
	    retVal = "203";
	else {
	    FileItemFactory factory = new DiskFileItemFactory();
	    ServletFileUpload upload = new ServletFileUpload(factory);
	    try {
		List items = upload.parseRequest(request);

		Map fields = new HashMap();

		Iterator iter = items.iterator();
		while (iter.hasNext()) {
		    FileItem item = (FileItem) iter.next();
		    if (item.isFormField())
			fields.put(item.getFieldName(), item.getString());
		    else
			fields.put(item.getFieldName(), item);
		}
		FileItem uplFile = (FileItem) fields.get("NewFile");
		String fileNameLong = uplFile.getName();
		fileNameLong = fileNameLong.replace('\\', '/');
		String[] pathParts = fileNameLong.split("/");
		String filename = pathParts[pathParts.length - 1];

		String baseName = FilenameUtils.getBaseName(filename);// Utils.getNameWithoutExtension(filename);
		String ext = FilenameUtils.getExtension(filename); // Utils.getExtension(fileName);
		File pathToSave = new File(currentDirPath, filename);
		int counter = 1;
		while (pathToSave.exists()) {
		    newName = baseName + "(" + counter + ")" + "." + ext;
		    retVal = "201";
		    pathToSave = new File(currentDirPath, newName);
		    counter++;
		}
		uplFile.write(pathToSave);
	    } catch (Exception ex) {
		retVal = "203";
	    }

	}

	out.println("<script type=\"text/javascript\">");
	out.println("window.parent.frames['frmUpload'].OnUploadCompleted(" + retVal + ",'" + newName + "');");
	out.println("</script>");
	out.flush();
	out.close();
	
	logger.debug("Successfull ended #doPost.");
    }

    private void setCreateFolderResponse(String retValue, Node root, Document doc) {
	Element myEl = doc.createElement("Error");
	myEl.setAttribute("number", retValue);
	root.appendChild(myEl);
    }

    private void getFolders(File dir, Node root, Document doc) {
	Element folders = doc.createElement("Folders");
	root.appendChild(folders);
	File[] fileList = dir.listFiles((FileFilter) DirectoryFileFilter.DIRECTORY);
	for (File file : fileList) {
	    Element myEl = doc.createElement("Folder");
	    myEl.setAttribute("name", file.getName());
	    folders.appendChild(myEl);
	}
    }

    private void getFiles(File dir, Node root, Document doc) {
	Element files = doc.createElement("Files");
	root.appendChild(files);
	File[] fileList = dir.listFiles((FileFilter) FileFileFilter.FILE);

	for (File file : fileList) {
	    Element myEl = doc.createElement("File");
	    myEl.setAttribute("name", file.getName());
	    myEl.setAttribute("size", String.valueOf(file.length() / 1024));
	    files.appendChild(myEl);
	}
    }

    private Node CreateCommonXml(Document doc, String commandStr, String typeStr, String currentPath, String currentUrl) {
	Element root = doc.createElement("Connector");
	doc.appendChild(root);
	root.setAttribute("command", commandStr);
	root.setAttribute("resourceType", typeStr);

	Element myEl = doc.createElement("CurrentFolder");
	myEl.setAttribute("path", currentPath);
	myEl.setAttribute("url", currentUrl);
	root.appendChild(myEl);

	return root;
    }

}
